import { __awaiter, __decorate } from "tslib";
import { Connect } from '@genesislcap/foundation-comms';
import { ZeroAgSelectRenderer, ZeroAgTextRenderer } from '@genesislcap/foundation-zero-grid-pro';
import { customElement, FASTElement, observable } from '@microsoft/fast-element';
import { filterObjectArray, getFiltersByFieldType } from '../../utils';
import { AVAILABLE_FIELDS_CONFIG, SELECTED_FILTERS_CONFIG } from './columns';
import { SelectFiltersStyles } from './select-filters.styles';
import { SelectFiltersTemplate } from './select-filters.template';
let SelectFilters = class SelectFilters extends FASTElement {
    constructor() {
        super(...arguments);
        this.filtersToRemove = [];
        this.selectedFields = [];
        this.selectedFilters = [];
        this.filterGridReady = false;
        // TODO: extract to util
        this.makeSureFieldValueIsNotLost = (array) => {
            return array.map((field) => {
                var _a;
                const matchingField = (_a = this.selectedFilters) === null || _a === void 0 ? void 0 : _a.find((f) => (f === null || f === void 0 ? void 0 : f.name) === (field === null || field === void 0 ? void 0 : field.name));
                return Object.assign(Object.assign({}, field), matchingField);
            });
        };
        this.onFilterSelect = (value, params) => {
            const newFields = this.selectedFilters.map((field) => {
                if (field.name === params.node.data.name) {
                    field.filter.conditional = value;
                }
                return field;
            });
            this.setSelectedFilters(newFields);
        };
        this.onFilterValueChange = (value, params) => {
            const newFields = this.selectedFilters.map((field) => {
                if (field.name === params.node.data.name) {
                    field.filter.value = value;
                }
                return field;
            });
            this.setSelectedFilters(newFields);
        };
        /**
         * Remove fields from selected filters grid and deselect them from the available fields grid
         */
        this.removeSelectedFilters = () => {
            // filter fields from selected list
            const filters = filterObjectArray('name', this.selectedFilters, this.filtersToRemove);
            this.setSelectedFilters(filters);
            // reset removal array
            this.filtersToRemove = [];
            // deselect available fieldgrid
            this.availableFieldsGrid.gridApi.deselectAll();
            // reselect
            this.availableFieldsGrid.gridApi.forEachNode((node) => {
                filters.find((field) => field.name === node.data.name) && node.setSelected(true);
            });
        };
    }
    connectedCallback() {
        super.connectedCallback();
        this.loadAvailableFieldsGrid();
        this.loadSelectedFiltersGrid();
    }
    selectedFiltersChanged() {
        if (this.filterGridReady) {
            this.selectedFiltersGrid.gridApi.setRowData(this.selectedFilters);
            this.selectedFiltersGrid.gridApi.refreshCells();
        }
    }
    /**
     * Update selected filters on parent component
     */
    setSelectedFilters(filters) {
        this.$emit('setSelectedFilters', { filters });
    }
    /**
     * Get a simplified / normalized version of the filters
     * e.g. ['REPORT_ID', 'REPORT_NAME']
     * @returns Array of string
     */
    getNormalizedFilters() {
        var _a;
        return (_a = this.selectedFilters) === null || _a === void 0 ? void 0 : _a.map(({ FIELD_NAME }) => FIELD_NAME);
    }
    /**
     * Loads availableFieldsGrid with data from the field selection step
     */
    loadAvailableFieldsGrid() {
        return __awaiter(this, void 0, void 0, function* () {
            this.availableFieldsGrid.gridOptions = Object.assign(Object.assign({}, AVAILABLE_FIELDS_CONFIG.gridOptions), { columnDefs: AVAILABLE_FIELDS_CONFIG.colDefs, rowData: [...this.selectedFields], onGridReady: () => {
                    window.addEventListener('resize', () => {
                        setTimeout(() => {
                            this.availableFieldsGrid.gridApi.sizeColumnsToFit();
                        });
                    });
                }, onFirstDataRendered: () => {
                    this.availableFieldsGrid.gridApi.sizeColumnsToFit();
                    this.selectDefaultFilters();
                }, onSelectionChanged: () => {
                    this.setSelectedFilters([
                        ...this.makeSureFieldValueIsNotLost(this.availableFieldsGrid.gridApi.getSelectedRows()),
                    ]);
                } });
        });
    }
    /**
     * Loads the selectedFiltersGrid
     */
    loadSelectedFiltersGrid() {
        this.selectedFiltersGrid.gridOptions = Object.assign(Object.assign({}, SELECTED_FILTERS_CONFIG.gridOptions), { columnDefs: [
                ...SELECTED_FILTERS_CONFIG.colDefs,
                {
                    field: 'TYPE',
                    headerName: 'Filter',
                    maxWidth: 170,
                    cellStyle: {
                        overflow: 'visible',
                        paddingLeft: 0,
                        paddingRight: 0,
                    },
                    cellRenderer: ZeroAgSelectRenderer,
                    cellRendererParams: {
                        accessor: 'filter.conditional',
                        defaultValue: 'Select filter',
                        onSelect: this.onFilterSelect,
                        options: (data) => getFiltersByFieldType(data.type),
                    },
                },
                {
                    cellRenderer: ZeroAgTextRenderer,
                    cellRendererParams: {
                        onChange: this.onFilterValueChange,
                        accessor: 'filter.value',
                    },
                    field: 'filter.value',
                    headerName: 'Value',
                },
            ], rowData: [...this.filtersToRemove, ...this.selectedFilters], onGridReady: () => {
                this.filterGridReady = true;
                window.addEventListener('resize', () => {
                    setTimeout(() => {
                        this.selectedFiltersGrid.gridApi.sizeColumnsToFit();
                    });
                });
            }, onFirstDataRendered: () => {
                this.selectedFiltersGrid.gridApi.sizeColumnsToFit();
            }, onSelectionChanged: () => {
                this.filtersToRemove = this.selectedFiltersGrid.gridApi.getSelectedRows();
            } });
    }
    /**
     * Select default filters
     */
    selectDefaultFilters() {
        if (this.selectedFilters.length) {
            this.availableFieldsGrid.gridApi.forEachNode((node) => {
                this.selectedFilters.find((field) => field.name === node.data.name) &&
                    node.setSelected(true);
            });
            this.setSelectedFilters([
                ...this.makeSureFieldValueIsNotLost(this.availableFieldsGrid.gridApi.getSelectedRows()),
            ]);
        }
    }
};
__decorate([
    Connect
], SelectFilters.prototype, "connect", void 0);
__decorate([
    observable
], SelectFilters.prototype, "filtersToRemove", void 0);
__decorate([
    observable
], SelectFilters.prototype, "selectedFields", void 0);
__decorate([
    observable
], SelectFilters.prototype, "selectedFilters", void 0);
__decorate([
    observable
], SelectFilters.prototype, "filterGridReady", void 0);
SelectFilters = __decorate([
    customElement({
        name: 'reporting-select-filters',
        template: SelectFiltersTemplate,
        styles: SelectFiltersStyles,
    })
], SelectFilters);
export { SelectFilters };
